<?php
/**
 * Plugin name: The Events Calendar: Month View GigPress Integration
 * Description: Makes shows added through GigPress visible within month view.
 * License:     GPLv3 or later https://opensource.org/licenses/gpl-3.0.html
 */

/**
 * Handles adding GigPress show data to month view.
 */
class TEC_GigPress_Month_View {
	/**
	 * Container for TEC_GigPress_Month_View_Shows objects (one per
	 * month).
	 *
	 * @var array
	 */
	protected $show_lists = array();

	/**
	 * Setup integration with month view/single day templates.
	 */
	public function __construct() {
		add_filter( 'tribe_get_template_part_content', array( $this, 'add_gigpress_listings' ), 10, 2 );
	}

	/**
	 * For each day in month view, inject a list of any GigPress shows taking place
	 * on that same date.
	 *
	 * @param string $html
	 * @param string $template
	 *
	 * @return string
	 */
	public function add_gigpress_listings( $html, $template ) {
		if ( $template !== 'month/single-day.php' ) {
			return $html;
		}

		$today = tribe_events_get_current_month_day();
		$month = date_create( $today['date'] )->format( 'Y-m' );

		$shows_this_month = $this->get_show_list_for( $month );
		$shows_today      = $shows_this_month->get_shows_for( $today['date'] );

		if ( $shows_today ) {
			$show_list  = new TEC_GigPress_Month_View_Show_Lister( $shows_today );
			$html = $this->inject( $show_list, $html );
		}

		return $html;
	}

	/**
	 * Returns an object with data about GigPress shows for the specified month,
	 * which should provided in 'yyyy-mm' style format.
	 *
	 * @param $month
	 *
	 * @return TEC_GigPress_Month_View_Shows
	 */
	protected function get_show_list_for( $month ) {
		if ( empty( $this->show_lists[ $month ] ) ) {
			$this->show_lists[ $month ] = new TEC_GigPress_Month_View_Shows( $month );
		}

		return $this->show_lists[ $month ];
	}

	/**
	 * Injects the show list into the single day HTML.
	 *
	 * This depends on there being an "Events List" HTML comment (as in the default
	 * template) that the method can use to find a nice injection point.\
	 *
	 * @param TEC_GigPress_Month_View_Show_Lister $show_list
	 * @param string $html
	 *
	 * @return string
	 */
	protected function inject( TEC_GigPress_Month_View_Show_Lister $show_list, $html ) {
		// Look for this marker and inject the new list *after* it
		$marker = '<!-- Events List -->';
		$marker_position = strpos( $html, $marker );

		if ( false === $marker_position ) {
			return $html;
		}

		return substr_replace( $html, (string) $show_list, $marker_position + strlen( $marker ), 0 );
	}
}

/**
 * Provides a list of GigPress shows for a specific month.
 *
 * @internal
 */
class TEC_GigPress_Month_View_Shows {
	protected $month;
	protected $shows = array();
	protected $days  = array();

	/**
	 * @param string $month ("yyyy-mm" format)
	 */
	public function __construct( $month ) {
		// Gracefully bow out without doing anything if the month is not in the expected format
		if ( ! preg_match( '/[0-9]{4}-[0-9]{2}/', $month ) ) {
			return;
		}

		$this->month = $month;
		$this->load_shows();
		$this->analyze_shows();
	}

	/**
	 * Looks for shows taking place this month.
	 */
	protected function load_shows() {
		global $wpdb;

		$shows_table = GIGPRESS_SHOWS;

		try {
			$start_date = date_create( $this->month . '-01' );
			$end_date   = clone $start_date ;
		}
		catch ( Exception $e ) {
			return array();
		}

		$start_date = $start_date->format( 'Y-m-d H:i:s' );
		$end_date   = $end_date->modify( '+1 month -1 day' )->format( 'Y-m-d 23:59:59' );

		$query = "
			SELECT   *,
			         CONCAT( show_date, ' ', show_time ) AS start_time
			FROM     $shows_table
			WHERE    (
			             show_date >= %s AND
			             show_date <= %s
			         )
			ORDER BY start_time ASC
	    ";

		$this->shows = $wpdb->get_results( $wpdb->prepare(
			$query,
			$start_date,
			$end_date
		) );
	}

	/**
	 * Counts the number of shows taking place each day of the month,
	 * primarily as a fast means of checking if a given day has any
	 * shows.
	 */
	protected function analyze_shows() {
		foreach ( $this->shows as $single_show ) {
			$this->days[ $single_show->show_date ]++;
		}
	}

	/**
	 * Returns a possibly empty array of show objects for the specified day.
	 *
	 * @param string $date ("yyyy-mm-dd" format)
	 *
	 * @return array
	 */
	public function get_shows_for( $date ) {
		if ( ! isset( $this->days[ $date] ) ) {
			return array();
		}

		$accumulator = array();

		foreach ( $this->shows as $single_show ) {
			if ( $single_show->show_date === $date ) {
				$accumulator[] = $single_show;
			}
		}

		return $accumulator;
	}
}

/**
 * Transforms a TEC_GigPress_Month_View_Shows object into an HTML
 * representation for inclusion in month view.
 *
 * @internal
 */
class TEC_GigPress_Month_View_Show_Lister {
	protected $shows = array();
	protected $html  = '';

	public function __construct( array $shows ) {
		$this->shows = $shows;
		$this->render();
	}

	protected function render() {
		foreach ( $this->shows as $single_show ) {
			$artist = $this->get_artist( $single_show->show_artist_id );
			$venue  = $this->get_venue( $single_show->show_venue_id );

			if ( is_null( $artist ) ) {
				continue;
			}

			/* The classes and ID mimic those of a regular event within month view/single day
			 * for better visual assimilation (but still allow them to be picked out and
			 * styled separately)
			 */
			$id = esc_attr( 'tribe-events-event-gigpress-' . $artist->artist_id );
			$classes = esc_attr( 'tribe_events type-tribe_events gigpress_listing hentry' );

			$gigpress_show_html = "<div id='$id' class='$classes'>"
				. '<h3 class="tribe-events-month-event-title">'
				. $this->get_artist_chunk( $artist )
				. $this->get_venue_chunk( $venue )
				. '</h3> </div>';

			/**
			 * @var string $gigpress_show_html
			 * @var object $artist
			 * @var object $venue
			 */
			$this->html .= apply_filters( 'tec_gigpress_month_view_listing', $gigpress_show_html, $artist, $venue );
		}
	}

	protected function get_artist( $artist_id ) {
		global $wpdb;

		$gigpress_table = GIGPRESS_ARTISTS;

		return $wpdb->get_row( $wpdb->prepare( "
				SELECT *
				FROM   $gigpress_table
				WHERE  artist_id = %d
			",
			$artist_id
		) );
	}

	protected function get_venue( $venue_id ) {
		global $wpdb;

		$gigpress_table = GIGPRESS_VENUES;

		return $wpdb->get_row( $wpdb->prepare( "
				SELECT *
				FROM   $gigpress_table
				WHERE  venue_id = %d
			",
			$venue_id
		) );
	}

	protected function get_artist_chunk( $artist ) {
		$artist_html = $artist->artist_name;

		if ( ! empty( $artist->artist_url ) ) {
			$artist_html = '<a href="' . esc_url( $artist->artist_url ) . '" target="_blank">' . $artist_html . '</a>';
		}

		return $artist_html;
	}

	protected function get_venue_chunk( $venue ) {
		if ( null === $venue ) {
			return '';
		}

		$venue_html = $venue->venue_name;

		if ( ! empty( $venue->venue_url ) ) {
			$venue_html = '<a href="' . esc_url( $venue->venue_url ) . '" target="_blank">' . $venue_html . '</a>';
		}

		return " @ $venue_html";
	}

	public function __toString() {
		return $this->html;
	}
}

/**
 * @return TEC_GigPress_Month_View
 */
function gigpress_month_view_integration() {
	static $object = null;

	if ( empty( $object ) ) {
		$object = new TEC_GigPress_Month_View;
	}

	return $object;
}

gigpress_month_view_integration();